@@ -6,17 +6,17 @@
  * be adding a fixed offset based on a hidden epoch value, in which case
  * unhappy things will occur on the next rollover.
  *
- * This file is Copyright (c) 2010-2018 by the GPSD project
+ * This file is Copyright 2010 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  *
  */
 
 #include "gpsd_config.h"  /* must be before all includes */
 
-#include <stdio.h>
-#include <stdbool.h>
-#include <string.h>
 #include <math.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "gpsd.h"
@@ -42,7 +42,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
 {
     unsigned short flags, pflags;
     timespec_t ts_tow;
-    uint32_t tow;	     /* Time of week [ms] */
+    uint32_t tow;            /* Time of week [ms] */
     char ts_buf[TIMESPEC_LEN];
 
     gps_mask_t mask = 0;
@@ -57,7 +57,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
     //cflags = (unsigned short) getleu16(buf, 7 + 6);
     pflags = (unsigned short) getleu16(buf, 7 + 8);
 
-    session->gpsdata.status = STATUS_NO_FIX;
+    session->newdata.status = STATUS_NO_FIX;
     session->newdata.mode = MODE_NO_FIX;
     mask = ONLINE_SET | MODE_SET | STATUS_SET | CLEAR_IS;
 
@@ -109,9 +109,9 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
             session->newdata.mode = MODE_2D;
 
         if (pflags & FIX_FLAG_DGPS_CORRECTION)
-            session->gpsdata.status = STATUS_DGPS_FIX;
+            session->newdata.status = STATUS_DGPS_FIX;
         else
-            session->gpsdata.status = STATUS_FIX;
+            session->newdata.status = STATUS_FIX;
     }
 
     GPSD_LOG(LOG_DATA, &session->context->errout,
@@ -123,7 +123,7 @@ static gps_mask_t decode_itk_navfix(struct gps_device_t *session,
              session->newdata.ecef.y, session->newdata.ecef.z,
              session->newdata.altHAE, session->newdata.speed,
              session->newdata.track, session->newdata.climb,
-             session->newdata.mode, session->gpsdata.status,
+             session->newdata.mode, session->newdata.status,
              session->gpsdata.dop.gdop, session->gpsdata.dop.pdop,
              session->gpsdata.dop.hdop, session->gpsdata.dop.vdop,
              session->gpsdata.dop.tdop);
@@ -160,7 +160,8 @@ static gps_mask_t decode_itk_prnstatus(struct gps_device_t *session,
 
             flags = (unsigned short) getleu16(buf, off);
             used = (bool)(flags & PRN_FLAG_USE_IN_NAV);
-            session->gpsdata.skyview[st].PRN = (short)(getleu16(buf, off + 4) & 0xff);
+            session->gpsdata.skyview[st].PRN =
+                (short)(getleu16(buf, off + 4) & 0xff);
             session->gpsdata.skyview[st].elevation =
                 (double)(getles16(buf, off + 6) & 0xff);
             session->gpsdata.skyview[st].azimuth =
@@ -195,7 +196,7 @@ static gps_mask_t decode_itk_utcionomodel(struct gps_device_t *session,
     int leap;
     unsigned short flags;
     timespec_t ts_tow;
-    uint32_t tow;	     /* Time of week [ms] */
+    uint32_t tow;            /* Time of week [ms] */
     char ts_buf[TIMESPEC_LEN];
 
     if (len != 64) {
@@ -455,17 +456,14 @@ const struct gps_type_t driver_italk =
     .rtcm_writer    = gpsd_write,       /* send RTCM data straight */
     .init_query     = NULL,             /* non-perturbing initial query */
     .event_hook     = NULL,             /* lifetime event handler */
-#ifdef RECONFIGURE_ENABLE
     .speed_switcher = NULL,             /* no speed switcher */
     .mode_switcher  = NULL,             /* no mode switcher */
     .rate_switcher  = NULL,             /* no sample-rate switcher */
-    .min_cycle.tv_sec  = 1,		/* not relevant, no rate switch */
-    .min_cycle.tv_nsec = 0,		/* not relevant, no rate switch */
-#endif /* RECONFIGURE_ENABLE */
-#ifdef CONTROLSEND_ENABLE
+    .min_cycle.tv_sec  = 1,             /* not relevant, no rate switch */
+    .min_cycle.tv_nsec = 0,             /* not relevant, no rate switch */
     .control_send   = NULL,             /* no control string sender */
-#endif /* CONTROLSEND_ENABLE */
     .time_offset     = NULL,            /* no method for NTP fudge factor */
 };
 /* *INDENT-ON* */
 #endif /* defined(ITRAX_ENABLE) && defined(BINARY_ENABLE) */
+// vim: set expandtab shiftwidth=4
