@@ -44,8 +44,8 @@
  *         # gpsrinex -i 1 -n 1000000
  *
  * See also:
- *     [1] RINEX: The Receiver Independent Exchange Format, Version 3.03
- *     ftp://igs.org/pub/data/format/rinex303.pdf
+ *     [1] RINEX: The Receiver Independent Exchange Format, Version 3.04
+ *     ftp://igs.org/pub/data/format/rinex304.pdf
  *
  *     [2] GPSTk, http://www.gpstk.org/
  *
@@ -56,29 +56,36 @@
  *     [4] RTKLIB: An Open Source Program Package for GNSS Positioning
  *     http://www.rtklib.com/
  *
- * This file is Copyright (c) 2018 by the GPSD project
+ * This file is Copyright 2018 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  *
  */
 
 #include "gpsd_config.h"  /* must be before all includes */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <string.h>
-#include <math.h>
-#include <time.h>
+#include <assert.h>
 #include <errno.h>
 #include <libgen.h>
+#include <math.h>
 #include <signal.h>
-#include <assert.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/types.h>    // for umask()
+#include <sys/stat.h>     // for umask()
+#include <time.h>
 #include <unistd.h>
 
+#ifdef HAVE_GETOPT_LONG
+       #include <getopt.h>
+#endif
+
+#include "compiler.h"
 #include "gps.h"
 #include "gpsdclient.h"
-#include "revision.h"
 #include "os_compat.h"
+#include "timespec.h"
 
 static char *progname;
 static struct fixsource_t source;
@@ -88,8 +95,23 @@ static double ecefz = 0.0;
 static timespec_t start_time = {0};      /* report gen time, UTC */
 static timespec_t first_mtime = {0};     /* GPS time, not UTC */
 static timespec_t last_mtime = {0};      /* GPS time, not UTC */
+static int leap_seconds = 0;             // set if non-zero
 
-/* total count of observations by u-blox gnssid [0-6]
+// strings for the RINEX file
+static char agency[41] = "Unknown";
+static char ant_num[21] = "0";
+static char ant_type[21] = "UNKNOWN EXT     NONE";
+static double ant_e = 0.0;
+static double ant_h = 0.0;
+static double ant_n = 0.0;
+static char marker_name[61] = "XXXX";
+static char marker_type[61] = "NON_PHYSICAL";
+static char observer[21] = "Unknown";
+static char rec_num[21] = "0";
+static char rec_type[21] = "Unknown";
+static char rec_vers[21] = "0";
+
+/* total count of observations by u-blox gnssid [0-7]
  *  0 = GPS       RINEX G
  *  1 = SBAS      RINEX S
  *  2 = Galileo   RINEX E
@@ -145,8 +167,10 @@ static struct obs_cnt_t {
 
 static FILE * tmp_file;             /* file handle for temp file */
 static int sample_count = 20;       /* number of measurement sets to get */
-/* seconds between measurement sets */
-static unsigned int sample_interval = 30;
+/* timespec_t between measurement sets */
+static timespec_t sample_interval_ts = {30, 0};
+/* milli-seconds between measurement sets */
+static unsigned  sample_interval_ms = 30000;
 
 #define DEBUG_QUIET 0
 #define DEBUG_INFO 1
@@ -236,10 +260,10 @@ static int compare_obs_cnt(const void  *A, const void  *B)
 
     /* 0 = svid means unused, make those last */
     if (0 == a->svid) {
-	a_gnssid = 255;
+        a_gnssid = 255;
     }
     if (0 == b->svid) {
-	b_gnssid = 255;
+        b_gnssid = 255;
     }
     if (a_gnssid != b_gnssid) {
         return a_gnssid - b_gnssid;
@@ -286,6 +310,7 @@ static void print_rinex_header(void)
     struct tm *report_time;
     struct tm *first_time;
     struct tm *last_time;
+    struct tm tm_buf;            // temp buffer for gmtime_r()
     int cnt;                     /* number of obs for one sat */
     int prn_count[GNSSID_CNT] = {0};   /* count of PRN per gnssid */
 
@@ -293,7 +318,7 @@ static void print_rinex_header(void)
         (void)fprintf(stderr, "doing header\n");
     }
 
-    report_time = gmtime(&(start_time.tv_sec));
+    report_time = gmtime_r(&(start_time.tv_sec), &tm_buf);
     (void)strftime(tmstr, sizeof(tmstr), "%Y%m%d %H%M%S UTC", report_time);
 
     (void)fprintf(log_file,
@@ -301,29 +326,29 @@ static void print_rinex_header(void)
         "3.03", "", "OBSERVATION DATA", "M: Mixed", "RINEX VERSION / TYPE");
     (void)fprintf(log_file,
         "%-20s%-20s%-20s%-20s\n",
-        "gpsrinex 3.20", "", tmstr,
+        "gpsrinex " VERSION, "", tmstr,
         "PGM / RUN BY / DATE");
     (void)fprintf(log_file, "%-60s%-20s\n",
          "Source: gpsd live data", "COMMENT");
-    (void)fprintf(log_file, "%-60s%-20s\n", "XXXX", "MARKER NAME");
-    (void)fprintf(log_file, "%-60s%-20s\n", "NON_PHYSICAL", "MARKER TYPE");
+    (void)fprintf(log_file, "%-60s%-20s\n", marker_name, "MARKER NAME");
+    (void)fprintf(log_file, "%-60s%-20s\n", marker_type, "MARKER TYPE");
+    (void)fprintf(log_file, "%-20s%-40s%-20s\n",
+                  observer, agency, "OBSERVER / AGENCY");
     (void)fprintf(log_file, "%-20s%-20s%-20s%-20s\n",
-                  "Unknown", "Unknown", "", "OBSERVER / AGENCY");
+                  rec_num, rec_type, rec_vers, "REC # / TYPE / VERS");
     (void)fprintf(log_file, "%-20s%-20s%-20s%-20s\n",
-                  "0", "UNKNOWN", "0", "REC # / TYPE / VERS");
-    (void)fprintf(log_file, "%-20s%-20s%-20s%-20s\n",
-                  "0", "UNKNOWN EXT     NONE", "" , "ANT # / TYPE");
+                  ant_num, ant_type, "" , "ANT # / TYPE");
     if (isfinite(ecefx) &&
-	isfinite(ecefy) &&
-	isfinite(ecefz)) {
-	(void)fprintf(log_file, "%14.4f%14.4f%14.4f%18s%-20s\n",
-	    ecefx, ecefy, ecefz, "", "APPROX POSITION XYZ");
+        isfinite(ecefy) &&
+        isfinite(ecefz)) {
+        (void)fprintf(log_file, "%14.4f%14.4f%14.4f%18s%-20s\n",
+            ecefx, ecefy, ecefz, "", "APPROX POSITION XYZ");
     } else if (DEBUG_INFO <= debug) {
-	(void)fprintf(stderr, "INFO: missing ECEF\n");
+        (void)fprintf(stderr, "INFO: missing ECEF\n");
     }
 
     (void)fprintf(log_file, "%14.4f%14.4f%14.4f%18s%-20s\n",
-        0.0, 0.0, 0.0, "", "ANTENNA: DELTA H/E/N");
+        ant_h, ant_e, ant_n, "", "ANTENNA: DELTA H/E/N");
     (void)fprintf(log_file, "%6d%6d%48s%-20s\n", 1, 1,
          "", "WAVELENGTH FACT L1/2");
 
@@ -339,7 +364,7 @@ static void print_rinex_header(void)
     if (0 < prn_count[GNSSID_GPS]) {
         /* GPS, code G */
         (void)fprintf(log_file, "%c%5d%4s%4s%4s%4s%4s%4s%4s%4s%22s%-20s\n",
-             gnssid2rinex(GNSSID_GPS), 5, "C1C", "L1C", "D1C", "C2C", "L2C",
+             gnssid2rinex(GNSSID_GPS), 6, "C1C", "L1C", "D1C", "C2C", "L2C",
              "D2C", "", "", "", "SYS / # / OBS TYPES");
     }
     if (0 < prn_count[GNSSID_SBAS]) {
@@ -351,27 +376,28 @@ static void print_rinex_header(void)
     if (0 < prn_count[GNSSID_GAL]) {
         /* Galileo, E1, E5 aand E6 only, code E  */
         (void)fprintf(log_file, "%c%5d%4s%4s%4s%4s%4s%4s%4s%4s%22s%-20s\n",
-             gnssid2rinex(GNSSID_GAL), 3, "C1C", "L1C", "D1C", "C7Q",
+             gnssid2rinex(GNSSID_GAL), 6, "C1C", "L1C", "D1C", "C7Q",
              "L7Q", "D7Q", "", "", "", "SYS / # / OBS TYPES");
     }
     if (0 < prn_count[GNSSID_BD]) {
         /* BeiDou, BDS, code C */
         (void)fprintf(log_file, "%c%5d%4s%4s%4s%4s%4s%4s%4s%4s%22s%-20s\n",
-             gnssid2rinex(GNSSID_BD), 5, "C1C", "L1C", "D1C", "C7I", "L7I",
+             gnssid2rinex(GNSSID_BD), 6, "C1C", "L1C", "D1C", "C7I", "L7I",
              "D7I", "", "", "", "SYS / # / OBS TYPES");
     }
     if (0 < prn_count[GNSSID_QZSS]) {
         /* QZSS, code J */
         (void)fprintf(log_file, "%c%5d%4s%4s%4s%4s%4s%4s%4s%4s%22s%-20s\n",
-             gnssid2rinex(GNSSID_QZSS), 5, "C1C", "L1C", "D1C", "C2L",
+             gnssid2rinex(GNSSID_QZSS), 6, "C1C", "L1C", "D1C", "C2L",
              "L2L", "D2L", "", "", "", "SYS / # / OBS TYPES");
     }
     if (0 < prn_count[GNSSID_GLO]) {
         /* GLONASS, R */
         (void)fprintf(log_file, "%c%5d%4s%4s%4s%4s%4s%4s%4s%4s%22s%-20s\n",
-             gnssid2rinex(GNSSID_GLO), 5, "C1C", "L1C", "D1C", "C2C", "L2C",
+             gnssid2rinex(GNSSID_GLO), 6, "C1C", "L1C", "D1C", "C2C", "L2C",
              "D2C", "", "", "", "SYS / # / OBS TYPES");
     }
+    // FIXME: Add IRNSS...
 
     (void)fprintf(log_file, "%6d%54s%-20s\n", obs_cnt_prns(255),
                   "", "# OF SATELLITES");
@@ -394,92 +420,95 @@ static void print_rinex_header(void)
         switch (obs_cnt[i].gnssid) {
         case GNSSID_GPS:
             /* GPS, code G */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C2C],
-			  obs_cnt[i].obs_cnts[L2C],
-			  obs_cnt[i].obs_cnts[D2C],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C2C],
+                          obs_cnt[i].obs_cnts[L2C],
+                          obs_cnt[i].obs_cnts[D2C],
+                          "", "PRN / # OF OBS");
             break;
         case GNSSID_SBAS:
             /* SBAS, L1C and L5C, code S */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C5I],
-			  obs_cnt[i].obs_cnts[L5I],
-			  obs_cnt[i].obs_cnts[D5I],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C5I],
+                          obs_cnt[i].obs_cnts[L5I],
+                          obs_cnt[i].obs_cnts[D5I],
+                          "", "PRN / # OF OBS");
             break;
         case GNSSID_GAL:
             /* Galileo, code E */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C7Q],
-			  obs_cnt[i].obs_cnts[L7Q],
-			  obs_cnt[i].obs_cnts[D7Q],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C7Q],
+                          obs_cnt[i].obs_cnts[L7Q],
+                          obs_cnt[i].obs_cnts[D7Q],
+                          "", "PRN / # OF OBS");
             break;
         case GNSSID_BD:
             /* BeiDou, code C */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C7I],
-			  obs_cnt[i].obs_cnts[L7I],
-			  obs_cnt[i].obs_cnts[D7I],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C7I],
+                          obs_cnt[i].obs_cnts[L7I],
+                          obs_cnt[i].obs_cnts[D7I],
+                          "", "PRN / # OF OBS");
             break;
         case GNSSID_QZSS:
             /* QZSS, code J */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C2L],
-			  obs_cnt[i].obs_cnts[L2L],
-			  obs_cnt[i].obs_cnts[D2L],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C2L],
+                          obs_cnt[i].obs_cnts[L2L],
+                          obs_cnt[i].obs_cnts[D2L],
+                          "", "PRN / # OF OBS");
             break;
         case GNSSID_GLO:
             /* GLONASS, code R */
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  obs_cnt[i].obs_cnts[C2C],
-			  obs_cnt[i].obs_cnts[L2C],
-			  obs_cnt[i].obs_cnts[D2C],
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6u%6u%6u%18s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          obs_cnt[i].obs_cnts[C2C],
+                          obs_cnt[i].obs_cnts[L2C],
+                          obs_cnt[i].obs_cnts[D2C],
+                          "", "PRN / # OF OBS");
             break;
+        // FIXME: Add GNSSID_IRNSS, L5A
         default:
-	    (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6s%6s%24s%-20s\n",
-			  gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
-			  obs_cnt[i].obs_cnts[C1C],
-			  obs_cnt[i].obs_cnts[L1C],
-			  obs_cnt[i].obs_cnts[D1C],
-			  "", "",
-			  "", "PRN / # OF OBS");
+            (void)fprintf(log_file,"   %c%02d%6u%6u%6u%6s%6s%24s%-20s\n",
+                          gnssid2rinex(obs_cnt[i].gnssid), obs_cnt[i].svid,
+                          obs_cnt[i].obs_cnts[C1C],
+                          obs_cnt[i].obs_cnts[L1C],
+                          obs_cnt[i].obs_cnts[D1C],
+                          "", "",
+                          "", "PRN / # OF OBS");
         }
     }
 
+    (void)fprintf(log_file, "%-10s%50s%-20s\n",
+                  "DBHZ", "", "SIGNAL STRENGTH UNIT");
     (void)fprintf(log_file, "%10.3f%50s%-20s\n",
-                  (double)sample_interval, "", "INTERVAL");
+                  (double)sample_interval_ms / 1000.0, "", "INTERVAL");
 
     /* GPS time not UTC */
-    first_time = gmtime(&(first_mtime.tv_sec));
+    first_time = gmtime_r(&(first_mtime.tv_sec), &tm_buf);
     (void)fprintf(log_file, "%6d%6d%6d%6d%6d%5d.%07ld%8s%9s%-20s\n",
          first_time->tm_year + 1900,
          first_time->tm_mon + 1,
@@ -492,7 +521,7 @@ static void print_rinex_header(void)
          "TIME OF FIRST OBS");
 
     /* GPS time not UTC */
-    last_time = gmtime(&(last_mtime.tv_sec));
+    last_time = gmtime_r(&(last_mtime.tv_sec), &tm_buf);
     (void)fprintf(log_file, "%6d%6d%6d%6d%6d%5d.%07ld%8s%9s%-20s\n",
          last_time->tm_year + 1900,
          last_time->tm_mon + 1,
@@ -663,90 +692,94 @@ static void one_sig(struct meas_t *meas)
     default:
         (void)fprintf(stderr, "ERROR: one_sig() gnmssid %u unknown sigid %u\n",
                       gnssid, sigid);
-        /* FALLTHROUGH */
+        FALLTHROUGH
     case 0:
         /* L1C */
-	cxx = C1C;
-	lxx = L1C;
-	dxx = D1C;
+        cxx = C1C;
+        lxx = L1C;
+        dxx = D1C;
         break;
     case 2:
-	/* GLONASS L2 OF or BeiDou B2I D1 */
+        /* GLONASS L2 OF or BeiDou B2I D1 */
         if (GNSSID_BD == gnssid) {
             /* WAG */
-	    cxx = C7I;
-	    lxx = L7I;
-	    dxx = D7I;
+            cxx = C7I;
+            lxx = L7I;
+            dxx = D7I;
         } else {
-	    cxx = C2C;
-	    lxx = L2C;
-	    dxx = D2C;
+            cxx = C2C;
+            lxx = L2C;
+            dxx = D2C;
         }
         break;
     case 3:
-	/* GPS L2 or BD B2I D2 */
-	cxx = C2C;
-	lxx = L2C;
-	dxx = D2C;
+        /* GPS L2 or BD B2I D2 */
+        cxx = C2C;
+        lxx = L2C;
+        dxx = D2C;
         break;
     case 5:
-	/* QZSS L2C (L) */
-	cxx = C2L;
-	lxx = L2L;
-	dxx = D2L;
+        /* QZSS L2C (L) */
+        cxx = C2L;
+        lxx = L2L;
+        dxx = D2L;
         break;
     case 6:
-	/* Galileo E5 bQ */
-	cxx = C7Q;
-	lxx = L7Q;
-	dxx = D7Q;
+        /* Galileo E5 bQ */
+        cxx = C7Q;
+        lxx = L7Q;
+        dxx = D7Q;
         break;
     }
 
     /* map snr to RINEX snr flag [1-9] */
     if (0 == meas->snr) {
-	snr = 0;
+        snr = 0;
     } else if (12 > meas->snr) {
-	snr = 1;
+        snr = 1;
     } else if (18 >= meas->snr) {
-	snr = 2;
+        snr = 2;
     } else if (23 >= meas->snr) {
-	snr = 3;
+        snr = 3;
     } else if (29 >= meas->snr) {
-	snr = 4;
+        snr = 4;
     } else if (35 >= meas->snr) {
-	snr = 5;
+        snr = 5;
     } else if (41 >= meas->snr) {
-	snr = 6;
+        snr = 6;
     } else if (47 >= meas->snr) {
-	snr = 7;
+        snr = 7;
     } else if (53 >= meas->snr) {
-	snr = 8;
+        snr = 8;
     } else {
-	/* snr >= 54 */
-	snr = 9;
+        /* snr >= 54 */
+        snr = 9;
     }
 
-    /* check for slip */
-    /* FIXME: use actual interval */
-    if (meas->locktime < (sample_interval * 1000)) {
-	meas->lli |= 2;
+    /* check for slip
+     * FIXME: use actual interval
+     * locktime is in milliseconds
+     * sample_interval_ms is milli seconds */
+    if (meas->locktime < sample_interval_ms) {
+        meas->lli |= 2;
     }
 
     if (0 != isfinite(meas->pseudorange)) {
-	obs_cnt_inc(gnssid, svid, cxx);
+        obs_cnt_inc(gnssid, svid, cxx);
     }
 
     if (0 != isfinite(meas->carrierphase)) {
-	obs_cnt_inc(gnssid, svid, lxx);
+        obs_cnt_inc(gnssid, svid, lxx);
     }
 
     if (0 != isfinite(meas->doppler)) {
-	obs_cnt_inc(gnssid, svid, dxx);
+        obs_cnt_inc(gnssid, svid, dxx);
     }
 
-    (void)fputs(fmt_obs(meas->pseudorange, 0, snr), tmp_file);
-    (void)fputs(fmt_obs(meas->carrierphase, meas->lli, 0), tmp_file);
+    (void)fputs(fmt_obs(meas->pseudorange, 0, 0), tmp_file);
+    // putting snr here, with phase, is deprecated.
+    // it should be an S observation.
+    (void)fputs(fmt_obs(meas->carrierphase, meas->lli, snr), tmp_file);
     (void)fputs(fmt_obs(meas->doppler, 0, 0), tmp_file);
 }
 
@@ -756,7 +789,8 @@ static void one_sig(struct meas_t *meas)
  */
 static void print_raw(struct gps_data_t *gpsdata)
 {
-    struct tm *tmp_now;
+    struct tm *now_time;
+    struct tm tm_buf;            // temp buffer for gmtime_r()
     unsigned nrec = 0;
     unsigned nsat = 0;
     unsigned i;
@@ -764,15 +798,36 @@ static void print_raw(struct gps_data_t *gpsdata)
     unsigned char last_svid = 0;
     int need_nl = 0;
     int got_l1 = 0;
+    time_t epoch_sec;
+    timespec_t interval_ts;
 
-    if ((last_mtime.tv_sec + (time_t)sample_interval) >
-        gpsdata->raw.mtime.tv_sec) {
+    TS_SUB(&interval_ts, &gpsdata->raw.mtime, &last_mtime);
+    if (!TS_GE(&interval_ts, &sample_interval_ts)) {
         /* not time yet */
         return;
     }
-    /* opus insists (time % interval) = 0 */
-    if (0 != (gpsdata->raw.mtime.tv_sec % sample_interval)) {
-        return;
+
+#ifdef __UNUSED
+    fprintf(stderr, "sample: %ld %ld\n", (long)sample_interval_ts.tv_sec,
+            (long)sample_interval_ts.tv_nsec);
+    fprintf(stderr, "epoch: %ld %ld\n", (long)gpsdata->raw.mtime.tv_sec,
+            (long)gpsdata->raw.mtime.tv_nsec);
+#endif // __UNUSED
+
+    // do modulo only for sample_interval of even seconds
+    if (0 == sample_interval_ts.tv_nsec &&
+        0 < sample_interval_ts.tv_sec) {
+        epoch_sec = gpsdata->raw.mtime.tv_sec;
+        if (500000000 < gpsdata->raw.mtime.tv_nsec) {
+             // round it up.  To match convbin.
+             // does this break opus?
+             epoch_sec++;
+        }
+
+        /* opus insists (time % interval) = 0 */
+        if (0 != (epoch_sec % sample_interval_ts.tv_sec)) {
+            return;
+        }
     }
 
     /* RINEX 3 wants records in each epoch sorted by gnssid.
@@ -835,14 +890,14 @@ static void print_raw(struct gps_data_t *gpsdata)
     }
 
     /* print epoch header line */
-    tmp_now = gmtime(&(last_mtime.tv_sec));
+    now_time = gmtime_r(&(last_mtime.tv_sec), &tm_buf);
     (void)fprintf(tmp_file,"> %4d %02d %02d %02d %02d %02d.%07ld  0%3u\n",
-         tmp_now->tm_year + 1900,
-         tmp_now->tm_mon + 1,
-         tmp_now->tm_mday,
-         tmp_now->tm_hour,
-         tmp_now->tm_min,
-         tmp_now->tm_sec,
+         now_time->tm_year + 1900,
+         now_time->tm_mon + 1,
+         now_time->tm_mday,
+         now_time->tm_hour,
+         now_time->tm_min,
+         now_time->tm_sec,
          (long)(last_mtime.tv_nsec / 100), nsat);
 
     last_gnssid = 0;
@@ -864,11 +919,11 @@ static void print_raw(struct gps_data_t *gpsdata)
         svid = gpsdata->raw.meas[i].svid;
         sigid = gpsdata->raw.meas[i].sigid;
 
-	if (DEBUG_RAW <= debug) {
-	    (void)fprintf(stderr,"record: %u:%u:%u %s\n",
+        if (DEBUG_RAW <= debug) {
+            (void)fprintf(stderr,"record: %u:%u:%u %s\n",
                           gnssid, svid, sigid,
-			  gpsdata->raw.meas[i].obs_code);
-	}
+                          gpsdata->raw.meas[i].obs_code);
+        }
 
         if (0 == gpsdata->raw.meas[i].svid) {
             /* should not happen... */
@@ -879,12 +934,12 @@ static void print_raw(struct gps_data_t *gpsdata)
         if ((last_gnssid != gpsdata->raw.meas[i].gnssid) ||
             (last_svid != gpsdata->raw.meas[i].svid)) {
 
-	    if (0 != need_nl) {
-		(void)fputs("\n", tmp_file);
-	    }
+            if (0 != need_nl) {
+                (void)fputs("\n", tmp_file);
+            }
             got_l1 = 0;
-	    /* new record line gnssid:svid preamble  */
-	    (void)fprintf(tmp_file,"%c%02d", rinex_gnssid, svid);
+            /* new record line gnssid:svid preamble  */
+            (void)fprintf(tmp_file,"%c%02d", rinex_gnssid, svid);
         }
 
         last_gnssid = gpsdata->raw.meas[i].gnssid;
@@ -899,38 +954,38 @@ static void print_raw(struct gps_data_t *gpsdata)
             break;
         case 2:
             /* GLONASS L2 OF or BD B2I D1 */
-	    if (0 == got_l1) {
-		/* space to start of L2 */
-		(void)fprintf(tmp_file, "%48s", "");
-	    }
+            if (0 == got_l1) {
+                /* space to start of L2 */
+                (void)fprintf(tmp_file, "%48s", "");
+            }
             one_sig(&gpsdata->raw.meas[i]);
             break;
         case 3:
             /* GPS L2 or BD B2I D2 */
-	    if (0 == got_l1) {
-		/* space to start of L2 */
-		(void)fprintf(tmp_file, "%48s", "");
-	    }
+            if (0 == got_l1) {
+                /* space to start of L2 */
+                (void)fprintf(tmp_file, "%48s", "");
+            }
             one_sig(&gpsdata->raw.meas[i]);
             break;
         case 5:
             /* QZSS L2C (L) */
-	    if (0 == got_l1) {
-		/* space to start of L2 */
-		(void)fprintf(tmp_file, "%48s", "");
-	    }
+            if (0 == got_l1) {
+                /* space to start of L2 */
+                (void)fprintf(tmp_file, "%48s", "");
+            }
             one_sig(&gpsdata->raw.meas[i]);
             break;
         case 6:
             /* Galileo E5 bQ */
-	    if (0 == got_l1) {
-		/* space to start of L2 */
-		(void)fprintf(tmp_file, "%48s", "");
-	    }
+            if (0 == got_l1) {
+                /* space to start of L2 */
+                (void)fprintf(tmp_file, "%48s", "");
+            }
             one_sig(&gpsdata->raw.meas[i]);
             break;
         default:
-	    (void)fprintf(stderr,
+            (void)fprintf(stderr,
                           "ERROR: print_raw() gnssid %u unknown sigid %u\n",
                           gnssid, sigid);
             break;
@@ -963,10 +1018,21 @@ static void quit_handler(int signum)
  */
 static void conditionally_log_fix(struct gps_data_t *gpsdata)
 {
+    if (0 == leap_seconds && 0 < gpsdata->leap_seconds) {
+        // grab a static copy of the current leap second.
+        leap_seconds = gpsdata->leap_seconds;
+    }
+
     if (DEBUG_PROG <= debug) {
         /* The (long long unsigned) is for 32/64-bit compatibility */
-        (void)fprintf(stderr, "mode %d set %llx\n", gpsdata->fix.mode,
-                      (long long unsigned)gpsdata->set);
+        (void)fprintf(stderr, "mode %d set %llx leap %d\n",
+                      gpsdata->fix.mode,
+                      (long long unsigned)gpsdata->set,
+                      leap_seconds);
+    }
+    if (0 == leap_seconds) {
+        // Can't do anything until we know the current leap second
+        return;
     }
 
     /* mostly we don't care if 2D or 3D fix, let the post processor
@@ -974,7 +1040,7 @@ static void conditionally_log_fix(struct gps_data_t *gpsdata)
 
     if (MODE_2D < gpsdata->fix.mode) {
         /* got a good 3D fix */
-	if (1.0 > ecefx &&
+        if (1.0 > ecefx &&
             isfinite(gpsdata->fix.ecef.x) &&
             isfinite(gpsdata->fix.ecef.y) &&
             isfinite(gpsdata->fix.ecef.z)) {
@@ -983,9 +1049,9 @@ static void conditionally_log_fix(struct gps_data_t *gpsdata)
             ecefy = gpsdata->fix.ecef.y;
             ecefz = gpsdata->fix.ecef.z;
 
-	    if (DEBUG_PROG <= debug) {
-		(void)fprintf(stderr,"got ECEF\n");
-	    }
+            if (DEBUG_PROG <= debug) {
+                (void)fprintf(stderr,"got ECEF\n");
+            }
         }
     }
 
@@ -993,6 +1059,11 @@ static void conditionally_log_fix(struct gps_data_t *gpsdata)
         if (DEBUG_RAW <= debug) {
             (void)fprintf(stderr,"got RAW\n");
         }
+        /* RINEX 3 prefers GPS time. Accepts GLO (UTC) time.
+         * NRCan does not accept GLO time
+         * Remove the leap second to get GPS from UTC.
+         */
+        gpsdata->raw.mtime.tv_sec += leap_seconds;
         print_raw(gpsdata);
     }
     return;
@@ -1005,20 +1076,52 @@ static void usage(void)
 {
     (void)fprintf(stderr,
           "Usage: %s [OPTIONS] [server[:port:[device]]]\n"
-          "     [-D debuglevel]   Set debug level, default 0\n"
-          "     [-f filename]     out to filename\n"
-          "                       gpsrinexYYYYDDDDHHMM.obs\n"
-          "     [-h]              print this usage and exit\n"
-          "     [-i interval]     time between samples, default: %d\n"
-          "     [-n count]        number samples to collect, default: %d\n"
-          "     [-V]              print version and exit\n"
           "\n"
-          "defaults to '%s -n %d -i %d localhost:2947'\n",
-          progname, sample_interval, sample_count, progname, sample_count,
-          sample_interval);
+          "Mandatory arguments to long options are mandatory for "
+          "short options too.\n"
+          "     -D, --debug debuglevel     Set debug level, default 0\n"
+          "     -f, --fileout  filename    out to filename\n"
+          "                                default: gpsrinexYYYYDDDDHHMM.obs\n"
+          "     -h, --help                 print this usage and exit\n"
+          "     -i, --interval interval    time between samples in seconds\n"
+          "                                default: %0.3f\n"
+          "     -n, --count count          number samples to collect\n"
+          "                                default: %d\n"
+          "     -V, --version              print version and exit\n"
+          "\nThese strings get placed in the generated RINEX 3 obs file\n"
+          "     --agency [agency]          agency\n"
+          "     --ant_e [easting]          antenna easting in meters\n"
+          "     --ant_h [height]           antenna height in meters\n"
+          "     --ant_n [northing]         antenna northing in meters\n"
+          "     --ant_num [num]            antenna number\n"
+          "     --ant_type [type]          antenna type\n"
+          "     --marker_name [name]       marker name\n"
+          "     --marker_type [type]       marker type\n"
+          "     --observer [observer]      observer\n"
+          "     --rec_num [num]            receiver number\n"
+          "     --rec_type [type]          receiver type\n"
+          "     --rec_vers [vers]          receiver vers\n"
+          "\n"
+          "defaults to '%s -n %d -i %0.3f localhost:2947'\n",
+          progname, (double)sample_interval_ms / 1000.0, sample_count, progname,
+          sample_count, (double)sample_interval_ms / 1000.0);
     exit(EXIT_FAILURE);
 }
 
+// defines for getopt_long()
+#define AGENCY 301
+#define ANT_E 302
+#define ANT_H 303
+#define ANT_N 304
+#define ANT_NUM 305
+#define ANT_TYPE 306
+#define MARKER_NAME 307
+#define MARKER_TYPE 308
+#define OBSERVER 309
+#define REC_NUM 310
+#define REC_TYPE 311
+#define REC_VERS 312
+
 /*
  *
  * Main
@@ -1026,32 +1129,75 @@ static void usage(void)
  */
 int main(int argc, char **argv)
 {
-    char tmstr[40];            /* time: YYYYDDDMMHH */
+    char tmstr[40];              // time: YYYYDDDMMHH
+    char tmp_fname[32];          // temp file name, for mkstemp
+    int tmp_file_desc;           // temp file descriptor
     struct tm *report_time;
+    struct tm tm_buf;            // temp buffer for gmtime_r()
     int ch;
     unsigned int flags = WATCH_ENABLE;
     char   *fname = NULL;
     int timeout = 10;
+    double f;
 
     progname = argv[0];
 
     log_file = stdout;
-    while ((ch = getopt(argc, argv, "D:f:hi:n:V")) != -1) {
+    while (1) {
+        const char *optstring = "D:f:hi:n:V";
+
+#ifdef HAVE_GETOPT_LONG
+        int option_index = 0;
+        static struct option long_options[] = {
+            {"agency", required_argument, NULL, AGENCY},
+            {"ant_num", required_argument, NULL, ANT_NUM},
+            {"ant_type", required_argument, NULL, ANT_TYPE},
+            {"ant_e", required_argument, NULL, ANT_E},
+            {"ant_h", required_argument, NULL, ANT_H},
+            {"ant_n", required_argument, NULL, ANT_N},
+            {"count", required_argument, NULL, 'n' },
+            {"debug", required_argument, NULL, 'D' },
+            {"fileout", required_argument, NULL, 'f' },
+            {"help", no_argument, NULL, 'h' },
+            {"interval", required_argument, NULL, 'i' },
+            {"marker_name", required_argument, NULL, MARKER_NAME},
+            {"marker_type", required_argument, NULL, MARKER_TYPE},
+            {"observer", required_argument, NULL, OBSERVER},
+            {"rec_num", required_argument, NULL, REC_NUM},
+            {"rec_type", required_argument, NULL, REC_TYPE},
+            {"rec_vers", required_argument, NULL, REC_VERS},
+            {"version", no_argument, NULL, 'V' },
+            {NULL, 0, NULL, 0},
+        };
+
+        ch = getopt_long(argc, argv, optstring, long_options, &option_index);
+#else
+        ch = getopt(argc, argv, optstring);
+#endif
+        if (ch == -1) {
+            break;
+        }
+
         switch (ch) {
         case 'D':
             debug = atoi(optarg);
             gps_enable_debug(debug, log_file);
             break;
-       case 'f':       /* Output file name. */
+        case 'f':       /* Output file name. */
             fname = strdup(optarg);
             break;
         case 'i':               /* set sampling interval */
-            sample_interval = (time_t) atoi(optarg);
-            if (sample_interval < 1)
-                sample_interval = 1;
-            if (sample_interval >= 3600)
+            f = safe_atof(optarg); // still in seconds
+            if (3600.0 <= f) {
                 (void)fprintf(stderr,
-                          "WARNING: saample interval is an hour or more!\n");
+                              "WARNING: sample interval is an hour or more!\n");
+            }
+            sample_interval_ms = (unsigned)(1000 * f); // now in ms
+            if (0 == sample_interval_ms) {
+                // underflow
+                sample_interval_ms = 1;
+            }
+            MSTOTS(&sample_interval_ts, sample_interval_ms);
             break;
         case 'n':
             sample_count = atoi(optarg);
@@ -1060,6 +1206,46 @@ int main(int argc, char **argv)
             (void)fprintf(stderr, "%s: version %s (revision %s)\n",
                           progname, VERSION, REVISION);
             exit(EXIT_SUCCESS);
+        case AGENCY:
+            strlcpy(agency, optarg, sizeof(agency));
+            break;
+        case ANT_E:
+            ant_e = safe_atof(optarg);
+            break;
+        case ANT_H:
+            ant_h = safe_atof(optarg);
+            break;
+        case ANT_N:
+            ant_n = safe_atof(optarg);
+            break;
+        case ANT_NUM:
+            strlcpy(ant_num, optarg, sizeof(ant_num));
+            break;
+        case ANT_TYPE:
+            strlcpy(ant_type, optarg, sizeof(ant_type));
+            break;
+        case MARKER_NAME:
+            strlcpy(marker_name, optarg, sizeof(marker_name));
+            break;
+        case MARKER_TYPE:
+            strlcpy(marker_type, optarg, sizeof(marker_type));
+            break;
+        case OBSERVER:
+            strlcpy(observer, optarg, sizeof(observer));
+            break;
+        case REC_NUM:
+            strlcpy(rec_num, optarg, sizeof(rec_num));
+            break;
+        case REC_TYPE:
+            strlcpy(rec_type, optarg, sizeof(rec_type));
+            break;
+        case REC_VERS:
+            strlcpy(rec_vers, optarg, sizeof(rec_vers));
+            break;
+        case 'h':
+            FALLTHROUGH
+        case '?':
+            FALLTHROUGH
         default:
             usage();
             /* NOTREACHED */
@@ -1076,13 +1262,19 @@ int main(int argc, char **argv)
         gpsd_source_spec(argv[optind], &source);
     }
     if (DEBUG_INFO <= debug) {
+        char *device;
+        if (NULL == source.device) {
+            device = "Default";
+        } else {
+            device = source.device;
+        }
         (void)fprintf(stderr, "INFO: server: %s port: %s  device: %s\n",
-                      source.server, source.port, source.device);
+                      source.server, source.port, device);
     }
 
     /* save start time of report */
     (void)clock_gettime(CLOCK_REALTIME, &start_time);
-    report_time = gmtime(&(start_time.tv_sec));
+    report_time = gmtime_r(&(start_time.tv_sec), &tm_buf);
 
     /* open the output file */
     if (NULL == fname) {
@@ -1115,9 +1307,19 @@ int main(int argc, char **argv)
         flags |= WATCH_DEVICE;
     (void)gps_stream(&gpsdata, flags, source.device);
 
-    tmp_file = tmpfile();
+    // create temp file, coverity does not like tmpfile()
+    // covarfity wants a umask
+    (void)umask(0177);        // force rw-r--r--
+    strlcpy(tmp_fname, "/tmp/gpsrinexXXXXXX", sizeof(tmp_fname));
+    tmp_file_desc = mkstemp(tmp_fname);
+    if (0 > tmp_file_desc) {
+        (void)fprintf(stderr, "ERROR: mkstemp(%s) failed: %s\n",
+                      tmp_fname, strerror(errno));
+        exit(2);
+    }
+    tmp_file = fdopen(tmp_file_desc, "w+");
     if (NULL == tmp_file) {
-        (void)fprintf(stderr, "ERROR: could not open temp file: %s\n",
+        (void)fprintf(stderr, "ERROR: fdopen() failed: %s\n",
                       strerror(errno));
         exit(2);
     }
@@ -1143,5 +1345,9 @@ int main(int argc, char **argv)
 
     print_rinex_footer();
 
+    // remove the temp file
+    (void)unlink(tmp_fname);
     exit(EXIT_SUCCESS);
 }
+
+// vim: set expandtab shiftwidth=4
