@@ -9,7 +9,7 @@ representations to libgps structures.
 
 PERMISSIONS
    Written by Eric S. Raymond, 2009
-   This file is Copyright (c) 2009-2018 by the GPSD project
+   This file is Copyright 2009 by the GPSD project
    SPDX-License-Identifier: BSD-2-clause
 
 ***************************************************************************/
@@ -35,7 +35,6 @@ PERMISSIONS
 static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
                          const char **endptr)
 {
-    int leapseconds; /* FIXME, unused... */
     int ret;
 
     const struct json_attr_t json_attrs_1[] = {
@@ -45,7 +44,7 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
                                  .len = sizeof(gpsdata->dev.path)},
         {"time",   t_time,    .addr.ts = &gpsdata->fix.time,
                                  .dflt.ts = {0, 0}},
-        {"leapseconds",   t_integer, .addr.integer = &leapseconds,
+        {"leapseconds",   t_integer, .addr.integer = &gpsdata->leap_seconds,
                                  .dflt.integer = 0},
         {"ept",    t_real,    .addr.real = &gpsdata->fix.ept,
                                  .dflt.real = NAN},
@@ -105,7 +104,7 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
                                  .dflt.integer = MODE_NOT_SEEN},
         {"sep",    t_real,    .addr.real = &gpsdata->fix.sep,
                                  .dflt.real = NAN},
-        {"status", t_integer, .addr.integer = &gpsdata->status,
+        {"status", t_integer, .addr.integer = &gpsdata->fix.status,
                                  .dflt.integer = STATUS_FIX},
         {"relN", t_real,  .addr.real = &gpsdata->fix.NED.relPosN,
                                  .dflt.real = NAN},
@@ -127,7 +126,17 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
                                  .dflt.real = NAN},
         {"dgpsSta", t_integer, .addr.integer = &gpsdata->fix.dgps_station,
                                  .dflt.integer = -1},
-        // ignore unkown keys, for cross-version compatibility
+        {"wanglem", t_real, .addr.real = &gpsdata->fix.wanglem,
+                                 .dflt.real = NAN},
+        {"wangler", t_real, .addr.real = &gpsdata->fix.wangler,
+                                 .dflt.real = NAN},
+        {"wanglet", t_real, .addr.real = &gpsdata->fix.wanglet,
+                                 .dflt.real = NAN},
+        {"wspeedr", t_real, .addr.real = &gpsdata->fix.wspeedr,
+                                 .dflt.real = NAN},
+        {"wspeedt", t_real, .addr.real = &gpsdata->fix.wspeedt,
+                                 .dflt.real = NAN},
+        // ignore unknown keys, for cross-version compatibility
         {"", t_ignore},
 
         {NULL},
@@ -178,7 +187,8 @@ static int json_raw_read(const char *buf, struct gps_data_t *gpsdata,
                          const char **endptr)
 {
     int measurements;
-    double mtime_s, mtime_ns;
+    // initialized to shut up clang
+    double mtime_s = 0.0, mtime_ns = 0.0;
 
     const struct json_attr_t json_attrs_meas[] = {
         /* *INDENT-OFF* */
@@ -323,7 +333,8 @@ static int json_att_read(const char *buf, struct gps_data_t *gpsdata,
         {"mag_st",   t_character, .addr.character = &gpsdata->attitude.mag_st},
         {"pitch",    t_real,      .addr.real = &gpsdata->attitude.pitch,
                                      .dflt.real = NAN},
-        {"pitch_st", t_character, .addr.character = &gpsdata->attitude.pitch_st},
+        {"pitch_st", t_character,
+         .addr.character = &gpsdata->attitude.pitch_st},
         {"roll",     t_real,      .addr.real = &gpsdata->attitude.roll,
                                      .dflt.real = NAN},
         {"roll_st",  t_character, .addr.character = &gpsdata->attitude.roll_st},
@@ -372,19 +383,21 @@ static int json_devicelist_read(const char *buf, struct gps_data_t *gpsdata,
         /* *INDENT-OFF* */
         {"class",      t_check,      .dflt.check = "DEVICE"},
         {"path",       t_string,     STRUCTOBJECT(struct devconfig_t, path),
-                                        .len = sizeof(gpsdata->devices.list[0].path)},
-        {"activated",  t_time,       STRUCTOBJECT(struct devconfig_t, activated)},
-        {"activated",  t_real,       STRUCTOBJECT(struct devconfig_t, activated)},
+         .len = sizeof(gpsdata->devices.list[0].path)},
+        {"activated",  t_time,
+         STRUCTOBJECT(struct devconfig_t, activated)},
+        {"activated",  t_real,
+         STRUCTOBJECT(struct devconfig_t, activated)},
         {"flags",      t_integer,    STRUCTOBJECT(struct devconfig_t, flags)},
         {"driver",     t_string,     STRUCTOBJECT(struct devconfig_t, driver),
-                                        .len = sizeof(gpsdata->devices.list[0].driver)},
+         .len = sizeof(gpsdata->devices.list[0].driver)},
         {"hexdata",    t_string,     STRUCTOBJECT(struct devconfig_t, hexdata),
-                                        .len = sizeof(gpsdata->devices.list[0].hexdata)},
+         .len = sizeof(gpsdata->devices.list[0].hexdata)},
         {"subtype",    t_string,     STRUCTOBJECT(struct devconfig_t, subtype),
                             .len = sizeof(gpsdata->devices.list[0].subtype)},
         {"subtype1",   t_string,     STRUCTOBJECT(struct devconfig_t, subtype1),
                             .len = sizeof(gpsdata->devices.list[0].subtype1)},
-        {"native",     t_integer,    STRUCTOBJECT(struct devconfig_t, driver_mode),
+        {"native",     t_integer, STRUCTOBJECT(struct devconfig_t, driver_mode),
                                         .dflt.integer = -1},
         {"bps",        t_uinteger,   STRUCTOBJECT(struct devconfig_t, baudrate),
                                         .dflt.uinteger = DEVDEFAULT_BPS},
@@ -396,7 +409,7 @@ static int json_devicelist_read(const char *buf, struct gps_data_t *gpsdata,
                                         .dflt.real = NAN},
         {"mincycle",   t_real,       STRUCTOBJECT(struct devconfig_t, mincycle),
                                         .dflt.real = NAN},
-        // ignore unkown keys, for cross-version compatibility
+        // ignore unknown keys, for cross-version compatibility
         {"", t_ignore},
         {NULL},
         /* *INDENT-ON* */
@@ -430,8 +443,10 @@ static int json_version_read(const char *buf, struct gps_data_t *gpsdata,
                                     .len = sizeof(gpsdata->version.release)},
         {"rev",       t_string,  .addr.string  = gpsdata->version.rev,
                                     .len = sizeof(gpsdata->version.rev)},
-        {"proto_major", t_integer, .addr.integer = &gpsdata->version.proto_major},
-        {"proto_minor", t_integer, .addr.integer = &gpsdata->version.proto_minor},
+        {"proto_major", t_integer,
+         .addr.integer = &gpsdata->version.proto_major},
+        {"proto_minor", t_integer,
+         .addr.integer = &gpsdata->version.proto_minor},
         {"remote",    t_string,  .addr.string  = gpsdata->version.remote,
                                     .len = sizeof(gpsdata->version.remote)},
         {NULL},
@@ -578,7 +593,7 @@ int json_oscillator_read(const char *buf, struct gps_data_t *gpsdata,
 }
 
 // Test for JSON read status values that should be treated as a go-ahead
-// for further processing.  JSON_BADATTR - to allow JSON attributes uknown
+// for further processing.  JSON_BADATTR - to allow JSON attributes unknown
 // to this version of the library, for forward compatibility, is an obvious
 // thing to go here.
 #define PASS(n) (((n) == 0) || ((n) == JSON_ERR_BADATTR))
@@ -629,6 +644,13 @@ int libgps_json_unpack(const char *buf,
             gpsdata->set |= CLIMBERR_SET;
         if (gpsdata->fix.mode != MODE_NOT_SEEN)
             gpsdata->set |= MODE_SET;
+        if (0 != isfinite(gpsdata->fix.wanglem) ||
+            0 != isfinite(gpsdata->fix.wangler) ||
+            0 != isfinite(gpsdata->fix.wanglet) ||
+            0 != isfinite(gpsdata->fix.wspeedr) ||
+            0 != isfinite(gpsdata->fix.wspeedt)) {
+            gpsdata->set |= NAVDATA_SET;
+        }
         return FILTER(status);
     } else if (str_starts_with(classtag, "\"class\":\"GST\"")) {
         status = json_noise_read(buf, gpsdata, end);
@@ -751,3 +773,4 @@ int libgps_json_unpack(const char *buf,
 #endif /* SOCKET_EXPORT_ENABLE */
 
 /* libgps_json.c ends here */
+// vim: set expandtab shiftwidth=4
