@@ -1,14 +1,14 @@
 /*
- * This file is Copyright (c) 2010-2018 by the GPSD project
+ * This file is Copyright 2010 by the GPSD project
  * SPDX-License-Identifier: BSD-2-clause
  */
 
 #include "gpsd_config.h"  /* must be before all includes */
 
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <math.h>
 #include <time.h>
 
 #include "gpsd.h"
@@ -84,15 +84,17 @@ static void dbl_to_str(const char *fmt, double val, char *bufp, size_t len,
                        const char *suffix)
 {
     if (0 == isfinite(val)) {
-	(void)strlcat(bufp, ",", len);
+        (void)strlcat(bufp, ",", len);
     } else {
-	str_appendf(bufp, len, fmt, val);
+        str_appendf(bufp, len, fmt, val);
     }
     if (NULL != suffix) {
-	str_appendf(bufp, len, "%s", suffix);
+        str_appendf(bufp, len, "%s", suffix);
     }
 }
 
+/* GPS Quality Indicator values for xxGGA
+ * Almost, not quite, the same as STATUS_* from gps.h */
 #define FIX_QUALITY_INVALID 0
 #define FIX_QUALITY_GPS 1
 #define FIX_QUALITY_DGPS 2
@@ -118,7 +120,7 @@ void gpsd_position_fix_dump(struct gps_device_t *session,
     utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str), &tm);
 
     if (session->gpsdata.fix.mode > MODE_NO_FIX) {
-        switch(session->gpsdata.status) {
+        switch(session->gpsdata.fix.status) {
         case STATUS_NO_FIX:
             fixquality = FIX_QUALITY_INVALID;
             break;
@@ -135,7 +137,7 @@ void gpsd_position_fix_dump(struct gps_device_t *session,
             fixquality = FIX_QUALITY_RTK_FLT;
             break;
        case STATUS_DR:
-            // FALLTHROUGH
+            FALLTHROUGH
        case STATUS_GNSSDR:
             fixquality = FIX_QUALITY_DR;
             break;
@@ -166,7 +168,7 @@ void gpsd_position_fix_dump(struct gps_device_t *session,
         dbl_to_str("%.3f,", session->gpsdata.fix.geoid_sep, bufp, len, "M,");
         /* empty place holders for Age of correction data, and
          * Differential base station ID */
-	(void)strlcat(bufp, ",", len);
+        (void)strlcat(bufp, ",", len);
         nmea_add_checksum(bufp);
     }
 }
@@ -184,6 +186,7 @@ static void gpsd_transit_fix_dump(struct gps_device_t *session,
     char var_str[BUF_SZ];
     char *var_dir = "";
     struct tm tm;
+    char valid = 'V';
 
     utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str), &tm);
     if ('\0' != time_str[0]) {
@@ -197,16 +200,24 @@ static void gpsd_transit_fix_dump(struct gps_device_t *session,
         time2_str[0] = '\0';
     }
     if (0 != isfinite(session->gpsdata.fix.magnetic_var)) {
-	f_str(session->gpsdata.fix.magnetic_var, "%.1f", var_str),
-	var_dir = (session->gpsdata.fix.magnetic_var > 0) ? "E" : "W";
+        f_str(session->gpsdata.fix.magnetic_var, "%.1f", var_str),
+        var_dir = (session->gpsdata.fix.magnetic_var > 0) ? "E" : "W";
     } else {
         var_str[0] = '\0';
         var_dir = "";
     }
+
+    if (MODE_2D > session->gpsdata.fix.mode) {
+        valid = 'V';      // Status: Warning
+    } else {
+        valid = 'A';      // Status: Valid
+    }
+
+    // FIXME! add in "Mode Indicator" after var_dir
     (void)snprintf(bufp, len,
                    "$GPRMC,%s,%c,%s,%c,%s,%c,%s,%s,%s,%s,%s",
                    time_str,
-                   session->gpsdata.status ? 'A' : 'V',
+                   valid,
                    degtodm_str(session->gpsdata.fix.latitude, "%09.4f",
                                lat_str),
                    ((session->gpsdata.fix.latitude > 0) ? 'N' : 'S'),
@@ -265,6 +276,10 @@ static void gpsd_binary_satellite_dump(struct gps_device_t *session,
             /* bad azimuth, ignore */
             continue;
         }
+        if (0 == isfinite(session->gpsdata.skyview[i].ss)) {
+            /* bad snr, ignore */
+            continue;
+        }
         if (j % 4 == 0) {
             bufp2 = bufp + strlen(bufp);
             str_appendf(bufp, len,
@@ -309,6 +324,7 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
     if (session->device_type != NULL) {
         int i, j;
         int max_channels = session->device_type->channels;
+        int mode = session->gpsdata.fix.mode;
 
         /* GPGSA commonly has exactly 12 channels, enforce that as a MAX */
         if ( 12 < max_channels ) {
@@ -317,8 +333,14 @@ static void gpsd_binary_quality_dump(struct gps_device_t *session,
         }
 
         bufp2 = bufp + strlen(bufp);
-        (void)snprintf(bufp, len,
-                       "$GPGSA,%c,%d,", 'A', session->gpsdata.fix.mode);
+        // fix.mode can be zero, legal values are only 1, 2 or 3
+        if (MODE_NO_FIX > mode) {
+            mode = MODE_NO_FIX;    // 1
+        } else if (MODE_3D < mode) {
+            mode = MODE_3D;        // 3
+        }
+        // We don't know if we are manual (M) or automatic (A), force "A".
+        (void)snprintf(bufp, len, "$GPGSA,%c,%d,", 'A', mode);
         j = 0;
         for (i = 0; i < max_channels; i++) {
             if (session->gpsdata.skyview[i].used == true){
@@ -385,7 +407,8 @@ static void gpsd_binary_time_dump(struct gps_device_t *session,
 
     if (MODE_NO_FIX < session->newdata.mode &&
         0 <= session->gpsdata.fix.time.tv_sec) {
-        struct tm tm;
+        // shut up clang about uninitialized
+        struct tm tm = {0};
         char time_str[TIMESTR_SZ];
 
         utc_to_hhmmss(session->gpsdata.fix.time, time_str, sizeof(time_str),
@@ -585,5 +608,6 @@ void nmea_ais_dump(struct gps_device_t *session,
 }
 #endif /* AIVDM_ENABLE */
 
-
 /* pseudonmea.c ends here */
+
+// vim: set expandtab shiftwidth=4
